;> File name: FBUTI.SC
;>
;>  Script that contains interface procs to ease creation
;>  of floating BUTTON, RADIOBUTTON, and CHECKBOX windows.
;>  
;>  Each interface proc generates a call to FBUT_Create.
;\

;=================================================== FBUT_Button
proc FBUT_Button
; Create a floating button window to display over a workspace form
; or a canvas window.
;
; The window is invisible and inactive when it is created.
;
; Call FBUT_Visible to make it visible (or invisible)
; Call FBUT_Active to make it active (or inactive)
; Call FBUT_Move to change its location or parent.
;
;
; Note: You can use a hidden button as a quick way to
;       associate a hotkey with a function. The button
;       will not be in the visible portion of the screen.
;       (This may be easier than adding a hotkey to your
;       KEY event handler.)
;
; Return value:
; The button window's handle. Zero if the window is not created.
(
    x.parent      ; {I} Parent window.
  , x.row         ; {I} Originrow of window (Relative to parent)
  , x.col         ; {I} Origincol of window (Relative to parent)
  , x.wid         ; {I} Width of window.
                  ;     The width will be adjusted so that there
                  ;     is at least one space to the left and
                  ;     right of the label within the window.
  , x.label       ; {I} Label for button. Enclose hotkey in
                  ;     tildes "~".
                  ;     Leading and trailing spaces are removed
                  ;     from the label.
                  ;     The label will will be centered in the
                  ;     width specified in x.wid.
                  ;     If no tildes are found, the first char
                  ;     will be used as the hotkey.
  , x.keycode     ; {I} Keycode for label's hotkey.
                  ;     Leave blank to default to hotkey defined
                  ;     in the label.
  , x.proc        ; {I} Proc to execproc when button is pushed
  , x.active      ; {I} If TRUE, automatically activate window and
                  ;     make it visible.
)
private x, y, z
    , winhand
    , format.info
    , label.txt
    , hilite.pos
    , hilite.len

dynarray format.info[]
format.info[ "parent" ] = x.parent
format.info[ "row" ] = x.row
format.info[ "col" ] = x.col
format.info[ "width" ] = x.wid
format.info[ "height" ] = 1
format.info[ "procname" ] = x.proc
format.info[ "style" ] = 1
format.info[ "type" ] = "B"
format.info[ "active" ] = x.active
format.info[ "visible" ] = x.active

dynarray label.txt[]
label.txt[ 1 ] = x.label

dynarray hilite.pos[]
dynarray hilite.len[]

; Caller may specify a single keycode as a number
if type( x.keycode ) = "N" then
    x = x.keycode
    dynarray x.keycode[]
    x.keycode[ 1 ] = x
endif

winhand = FBUT_Create( format.info, label.txt, x.keycode
    , hilite.pos, hilite.len )

return winhand

endproc
wproc( "FBUT_Button" )

;=================================================== FBUT_CheckBox
proc FBUT_CheckBox
; Create a floating CheckBox window to display over a
; workspace form or a canvas window.
;
; The window is invisible and inactive when it is created.
;
; Call FBUT_Visible to make it visible (or invisible)
; Call FBUT_Active to make it active (or inactive)
; Call FBUT_Move to change its location or parent.
;
;
; Return value:
; The checkbox window's handle.
;
; Zero if the window is not created.
(
    x.parent      ; {I} Parent window.
  , x.row         ; {I} Originrow of window
  , x.col         ; {I} Origincol of window
  , x.wid         ; {I} Width of window.
                  ;     The width will be adjusted so the widest
                  ;     label will fit.
                  ;     The width will also be adjusted to leave
                  ;     room for " [ ] " to the left of each
                  ;     label and a space to the right.
  , x.label       ; {I} Dynarray containing the label for each
                  ;     checkbox choice.
                  ;     Use index "0" (zero) for the heading.
                  ;     Use indexes starting at 1 for labels.
                  ;     Leading and trailing spaces will be
                  ;     removed from each label.
                  ;     Enclose the hotkey in tildes "~". If no
                  ;     tildes are found, the first char will be
                  ;     used as the hotkey.
  , x.keycode     ; {I} Dynarray containing keycode for the
                  ;     corresponding label's hotkey.
                  ;     Leave blank to default to hotkey defined
                  ;     in the label.
  , x.proc        ; {I} Proc to execproc when an entry is selected.
                  ;     The proc is executed after the var named
                  ;     in x.varname is set.
  , x.varname     ; {I} Name of var to set if an entry is selected.
                  ;     It will be set to the index of the entry
                  ;     in x.label.
  , x.active      ; {I} If TRUE, automatically activate window and
                  ;     make it visible.
)
private x
    , winhand
    , format.info
    , hilite.pos
    , hilite.len

dynarray format.info[]

if type( x.label ) = "DY" and
   isassigned( x.label[ "0" ] ) then
    format.info[ "heading" ] = x.label[ 0 ]
    x.wid = max( len( x.label[ 0 ] ) + 2, x.wid )
    release vars x.label[ 0 ]
endif
format.info[ "parent" ] = x.parent
format.info[ "row" ] = x.row
format.info[ "col" ] = x.col
format.info[ "width" ] = x.wid
format.info[ "procname" ] = x.proc
format.info[ "varname" ] = x.varname
format.info[ "style" ] = 1
format.info[ "type" ] = "C"
format.info[ "active" ] = x.active
format.info[ "visible" ] = x.active

dynarray hilite.pos[]
dynarray hilite.len[]

winhand = FBUT_Create( format.info, x.label, x.keycode
    , hilite.pos, hilite.len )

return winhand

endproc
wproc( "FBUT_CheckBox" )

;=================================================== FBUT_Radio
proc FBUT_Radio
; Create a floating radiobutton window to display over a
; workspace form or a canvas window.
;
; The window is invisible and inactive when it is created.
;
; Call FBUT_Visible to make it visible (or invisible)
; Call FBUT_Active to make it active (or inactive)
; Call FBUT_Move to change its location or parent.
;
;
; Return value:
; The radiobutton window's handle.
;
; Zero if the window is not created.
(
    x.parent      ; {I} Parent window.
  , x.row         ; {I} Originrow of window
  , x.col         ; {I} Origincol of window
  , x.wid         ; {I} Width of window.
                  ;     The width will be adjusted so the widest
                  ;     label will fit.
                  ;     The width will also be adjusted to leave
                  ;     room for " ( ) " to the left of each
                  ;     label and a space to the right.
  , x.label       ; {I} Dynarray containing the label for each
                  ;     radiobutton choice.
                  ;     Use index "0" (zero) for the heading.
                  ;     Use indexes starting at 1 for labels.
                  ;     Leading and trailing spaces will be
                  ;     removed from each label.
                  ;     Enclose the hotkey in tildes "~". If no
                  ;     tildes are found, the first char will be
                  ;     used as the hotkey.
  , x.keycode     ; {I} Dynarray containing keycode for the
                  ;     corresponding label's hotkey.
                  ;     Leave blank to default to hotkey defined
                  ;     in the label.
  , x.proc        ; {I} Proc to execproc when an entry is selected.
                  ;     The proc is executed after the var named
                  ;     in x.varname is set.
  , x.varname     ; {I} Name of var to set if an entry is selected.
                  ;     It will be set to the index of the entry
                  ;     in x.label.
  , x.active      ; {I} If TRUE, automatically activate window and
                  ;     make it visible.
)
private x
    , winhand
    , format.info
    , hilite.pos
    , hilite.len

dynarray format.info[]

if type( x.label ) = "DY" and
   isassigned( x.label[ "0" ] ) then
    format.info[ "heading" ] = x.label[ 0 ]
    x.wid = max( len( x.label[ 0 ] ) + 2, x.wid )
    release vars x.label[ 0 ]
endif

format.info[ "parent" ] = x.parent
format.info[ "row" ] = x.row
format.info[ "col" ] = x.col
format.info[ "width" ] = x.wid
format.info[ "procname" ] = x.proc
format.info[ "varname" ] = x.varname
format.info[ "style" ] = 1
format.info[ "type" ] = "R"
format.info[ "active" ] = x.active
format.info[ "visible" ] = x.active

dynarray hilite.pos[]
dynarray hilite.len[]

winhand = FBUT_Create( format.info, x.label, x.keycode
    , hilite.pos, hilite.len )

return winhand

endproc
wproc( "FBUT_Radio" )

