;> procdoc.sc

;> Generate proc doc from script files. The script files are
;> expected to be in a specific format which is described
;> in the listing proc (below),

;> The caller must set the following vars before playing this
;> script:
;>
;> - procdoc.script     - The filename of the script to list
;>                        (including the .SC extension).
;>                        If the script is not in the working
;>                        directory, then supply the path.
;>
;> - procdoc.listing    - The name of the output file for the
;>                        listing. If the file is not in the 
;>                        working directory, then supply the path.
;>
;> - procdoc.append    - TRUE if the listing file exists and output
;>                       is to be appended.
;>                       FALSE if the listing file is to be created
;>                       or replaced.
;>
;> The caller should also ensure that ECHO is off or
;> processing will be very slow. We leave the choice
;> up to the caller - we don't mess with ECHO.
;\

; We enclose the code to do the listing in a proc so that
; the working vars can be declared as private.
; The proc has a weird name so it will be less likely to
; duplicate a proc name defined in the caller.

release procs ProcDoc!XXX

proc ProcDoc!XXX
; List documentation for procs
;
; The expected format of the script file containing procs
; whose documentation is to be listed is as follows:
;
;  - The PROC keyword for procs to be listed is in column 1.
;     If a proc is not to be listed, put the PROC keyword in
;     column 2 or beyond.
;  - Lines are listed until a right paren ")" is found in
;    column 1.
;
;  procs are expected to hane the following format. The "1"
;  indicates column 1.
;
;  1...5....0....5 and so on
;  proc procname
;  ; Description of proc
;  (
;     parameter list description
;  )
;     body of script
;  endproc

; In addition to proc documentation between PROC and ")", this
; script also lists lines that have ";>" in columns 1 and 2.
; The ";>" is replaced with "** " in the listing.
;
; If a line begins with ";\", it is replaced with 3 blank lines.

; Lines that are to be listed should be end before column 70.
; This applies to comments and code between "proc" and ")" and
; to comments that start with ";>".
;
; This allows room for margins if you decide to print the
; listing file. 
;
; Return value:
; None.
; The proc doc listing will be created or appended to.
(
; No parameters. However, the caller must define the vars:
;  - procdoc.script
;  - procdoc.listing
;  - procdoc.append
)
private x, y, z
	, last.line
	, l.state
	, curr.line
	, x.line
	, list.the.line
	, end.of.proc
	, start.of.proc

; Open script file and determine the number of lines
editor open procdoc.script

; Clear wordwrap so long lines don't fuzzle things up.
menu {options} {wordwrap} {clear}

; Get to the end of the file
ctrlpgdn

; You can't always trust info when the cursor is at the last 
; byte of the file. Backup a couple of bytes.
left left

; Get line number of last line.
editor info complete to x
last.line = x[ "curline" ]
release vars x

; Back to the top.
ctrlpgup

end.of.proc = false
start.of.proc = false
l.state = 0  ; Initial state
curr.line = 0

while true
	list.the.line = false
	curr.line = curr.line + 1
	if curr.line >= last.line then
		quitloop
	endif
	message "Generating proc doc for script: "
		, upper( procdoc.script )
		, " Line "
		, curr.line
		, " of "
		, last.line
	shiftpress "end"
	editor extract to x.line
	switch
	case substr( x.line, 1, 2 ) = ";>"  :
		; Lines starting with ;> are listed regardless of the
		; current state. The ";>" is removed.
		list.the.line = true
		x.line = "** " + substr( x.line, 3, len( x.line ) - 2 )
	case substr( x.line, 1, 2 ) = ";\\"  :
		; Lines starting with ;\\ are replaced with blank lines
		; regardless of the current state.
		list.the.line = true
		x.line = "\n\n\n"
	case l.state = 0 :
		; Skip lines until we find the "proc" keyword in col 1
		if lower( substr( x.line, 1, 4 ) ) = "proc" and
		   ( substr( x.line, 5, 1 ) = " " or
		     substr( x.line, 5, 1 ) = "\n" ) then
		   
			list.the.line = true
			start.of.proc = true
			l.state = 1
		endif
	case l.state = 1 :
		; List lines until we find ")" in col 1
		list.the.line = true
		if lower( substr( x.line, 1, 1 ) ) = ")" then
			end.of.proc = true
			l.state = 0
		endif
	endswitch
	if list.the.line then
		if start.of.proc then
			; nothing special (yet)
			start.of.proc = false
		endif
		if procdoc.append then
			filewrite append procdoc.listing from x.line + "\n"
		else
			filewrite procdoc.listing from x.line + "\n"
			procdoc.append = true
		endif
		if end.of.proc then
			; Add some blank lines between scripts
			end.of.proc = false
			filewrite append procdoc.listing from "\n\n\n"
		endif
	endif
	right ; move to beginning of next line
endwhile
; Cancel the editor
menu {cancel}
while menuchoice() = "No" {yes} endwhile
message ""

endproc

; See note at top of script regarding vars that must be
; defined by the caller.

ProcDoc!XXX()  ;

; We're done with the proc, so we can ditch it and free the
; memory it occupies.

release procs ProcDoc!XXX

savevars all

